using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;
using System.Diagnostics;

namespace M3uExpand
{
	public partial class Form1 : Form
	{
		public Form1()
		{
			InitializeComponent();

			FilenameFormatCombo.SelectedIndex = 0;
		}

		private void Form1_DragOver(object sender, DragEventArgs e)
		{
			if (e.Data.GetDataPresent("FileDrop") && StartButton.Enabled) e.Effect = DragDropEffects.All;
		}

		private void Form1_DragDrop(object sender, DragEventArgs e)
		{
			if (e.Data.GetDataPresent("FileDrop") && StartButton.Enabled)
			{
				string[] files = (string[]) e.Data.GetData("FileDrop");
				M3uPlaylistLabel.Text = files[0];
				LoadPlaylistFile();
			}
		}

		private void LoadPlaylistFile()
		{
			FilesList.Items.Clear();
			int notFound = 0;
			StreamReader sr = new StreamReader(M3uPlaylistLabel.Text, Encoding.Default);
			while (!sr.EndOfStream)
			{
				string line = sr.ReadLine();
				if (!line.Trim().StartsWith("#"))
				{
					string s = line.Trim();
					string srcname = Path.Combine(Path.GetDirectoryName(M3uPlaylistLabel.Text), s);
					if (!File.Exists(srcname))
					{
						s += " [not found]";
						notFound++;
					}
					FilesList.Items.Add(s);
				}
			}
			sr.Close();
			if (notFound > 0)
			{
				MessageBox.Show(notFound + " files from the playlist do not exist.", "Missing files", MessageBoxButtons.OK, MessageBoxIcon.Warning);
			}
		}

		private void StartButton_Click(object sender, EventArgs e)
		{
			StartButton.Enabled = false;
			
			long totalsize = 0;
			progressBar1.Value = 0;
			foreach (string s in FilesList.Items)
			{
				string srcname = Path.Combine(Path.GetDirectoryName(M3uPlaylistLabel.Text), s);
				if (File.Exists(srcname))
					totalsize += new FileInfo(srcname).Length;
			}
			progressBar1.Maximum = (int) totalsize;

			int index = 1;
			foreach (string s in FilesList.Items)
			{
				string filename = Path.GetFileNameWithoutExtension(s);
				string artist = "";
				string title = filename;
				Match m = Regex.Match(filename, "^(.*?) - (.*)$");
				if (m.Success)
				{
					artist = m.Groups[1].Value;
					title = m.Groups[2].Value;
				}
				
				// Format new filename
				string newname = FilenameFormatCombo.Text;
				newname = newname.Replace("{m3uname}", Path.GetFileNameWithoutExtension(M3uPlaylistLabel.Text));
				newname = newname.Replace("{index}", index.ToString());
				newname = newname.Replace("{index:2}", index.ToString("00"));
				newname = newname.Replace("{index:3}", index.ToString("000"));
				newname = newname.Replace("{filename}", filename);
				newname = newname.Replace("{artist}", artist);
				newname = newname.Replace("{title}", title);

				newname = Path.Combine(Path.GetDirectoryName(M3uPlaylistLabel.Text), newname + Path.GetExtension(s));

				string srcname = Path.Combine(Path.GetDirectoryName(M3uPlaylistLabel.Text), s);

				// Copy file
				if (File.Exists(srcname))
				{
					Directory.CreateDirectory(Path.GetDirectoryName(newname));
					File.Copy(srcname, newname, true);
					progressBar1.Value += (int) new FileInfo(srcname).Length;
				}

				index++;
			}

			progressBar1.Value = 0;
	
			StartButton.Enabled = true;
		}

		private void HomepageLinklabel_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			Process.Start("http://beta.unclassified.de/projekte/m3uexpand/");
		}
	}
}